// Class CDraughtsEngineApp,	Version 4.0, June 2008
// Class CDraughtsDialog,		Version 4.0, June 2008

#include <afxwin.h>
#include <afxmt.h>		// Mutex
#include <afxcmn.h>		// CRichEditCtrl
#include <afxsock.h>	// CSocket Class
#include <omp.h>		// OpenMP
#include <MmSystem.h>	// TimeSetEvent()
#include <direct.h>		// _mkdir

#include "CGuide.h"
#include "CEngineDefinitions.h"
#include "CengineDef.h"
#include "CEngine.h"
#include "Led.h"
#include "pdn.h"
#include "DraughtsEngineApp.h"
#include "resource.h"

CDraughtsEngineApp  myApp ;


BOOL CDraughtsEngineApp::InitInstance()
{
	bool bGuideMode, bInstance, bProfile ;
	int iNCharFind, iNWhiteSpace0, iNWhiteSpace1, iPort ;
	CString csCmdLine, csIPAddress, csPort, csDamageMutex ;

	m_bShowWindow	= false ;
	bGuideMode		= false ;

	// Check number of instances
	m_iInstanceCount	= 0 ;
	bInstance			= true ;

	do {

		SetLastError(ERROR_SUCCESS) ;

		csDamageMutex.Format("DamageMutex%.2d", m_iInstanceCount) ;

		CreateMutex(NULL, TRUE, csDamageMutex) ;

		if ( GetLastError() == ERROR_ALREADY_EXISTS )			// Application is already running
			++m_iInstanceCount ;
		else
			bInstance = false ;

	} while ( bInstance ) ;

	// Initial No Communication
	m_iCommunicationMode = ENGINE_COMMUNICATIONMODE_GUIDE ;			// Temporary HACK, initially _NONE

	iPort		= DXP_PORT ;
	csIPAddress = DXP_IPADDRESS ;

	// Process m_lpCmdLine
	csCmdLine.Format("%s",m_lpCmdLine) ;
	csCmdLine.TrimLeft() ;

	iNCharFind = csCmdLine.Find("-guide") ;							// Check for -guide
	if ( iNCharFind != -1 )		
		bGuideMode = true ;

	iNCharFind = csCmdLine.Find("-showwindow") ;					// Check for -showwindow
	if ( iNCharFind != -1 )		
		m_bShowWindow = true ;

	iNCharFind		= csCmdLine.Find("-port") ;						// Check for Port Number

	if ( iNCharFind != -1 ) {
		iNWhiteSpace0	= csCmdLine.Find(" ", iNCharFind) ;			// Find first whitespace
		iNWhiteSpace1	= csCmdLine.Find(" ", iNWhiteSpace0+1) ;	// Find second whitespace (if any !)
	}

	if ( iNCharFind != -1  && iNWhiteSpace0 != -1 ) {

		if ( iNWhiteSpace1 == -1 )
			csPort = csCmdLine.Right( csCmdLine.GetLength() - iNWhiteSpace0 - 1 ) ;
		else
			csPort = csCmdLine.Mid(iNWhiteSpace0 + 1, iNWhiteSpace1 - iNWhiteSpace0 - 1 ) ;

		iPort = atoi(csPort) ;
	}
	// End Process m_lpCmdLine

	if ( bGuideMode == false )
		m_bShowWindow = true ;

#ifdef _ENGINE_DAMAGE
	_chdir("S:") ;	// Temporary only for Damage Engine
	m_csApplicationPath.Format("S:") ;
#endif

#ifdef _ENGINE_HORIZON
	char filespec[_MAX_PATH+1] ;

	if ( !GetModuleFileName( AfxGetInstanceHandle(), filespec, _MAX_PATH ) )
		return false ;

	m_csApplicationPath	= GetPath( filespec) ;
#endif

	m_csBookPath		= m_csApplicationPath + PATH_DIRECTORY_BOOK ;
	m_csDatabasePath	= m_csApplicationPath + PATH_DIRECTORY_DATABASES ;
	m_csGamesPath		= m_csApplicationPath + PATH_DIRECTORY_GAMES ;
	m_csScriptsPath		= m_csApplicationPath + PATH_DIRECTORY_SCRIPTS ;
	m_csTablesPath		= m_csApplicationPath + PATH_DIRECTORY_TABLES ;

	
	m_pEngine	= new CEngine ;					// Engine Class
	m_pPDN		= new CPDN ;					// CPDN Class
	
	if ( m_pEngine == 0 ) {
		AfxMessageBox( "Engine initialization Error.", MB_ICONEXCLAMATION ) ;
		return false ;
	}

	m_pGuide	= new CGuide ;					// Guide (= Engine <-> GUI communication ) Class
	if ( m_pGuide == 0 )
		return false ;

	if ( bGuideMode ) {

		if ( m_pGuide->Init(GUIDE_SERVER, ENGINE_COMMUNICATIONMODE_GUIDE, iPort, csIPAddress) ) {

			m_iCommunicationMode	= ENGINE_COMMUNICATIONMODE_GUIDE ;
			m_bGuide				= true ;	// Connection to GUI 

		} else {

			m_bGuide = false ;
			Beep(1000,1000) ;					// Beep Error signal !
		}
	}

	// Copy DraughtsEngineApp Data to Engine
	m_pEngine->m_bGuide				= m_bGuide ;
	m_pEngine->m_pGuide				= m_pGuide ;

	m_pEngine->m_pPDN				= m_pPDN ;

	m_pEngine->m_csApplicationPath	= m_csApplicationPath ;
	m_pEngine->m_csBookPath			= m_csBookPath ;
	m_pEngine->m_csDatabasePath		= m_csDatabasePath ;
	m_pEngine->m_csGamesPath		= m_csGamesPath ;
	m_pEngine->m_csScriptsPath		= m_csScriptsPath ;
	m_pEngine->m_csTablesPath		= m_csTablesPath ;

	m_pEngine->m_iCommunicationMode	= m_iCommunicationMode ;

	AfxInitRichEdit() ;		// Call this function to initialize the rich edit control for the application.
	
	m_pMainWnd	= new CDraughtsDialog ;

	if ( m_bShowWindow ) {
		m_pMainWnd->ShowWindow(m_nCmdShow) ;
		m_pMainWnd->UpdateWindow() ;
	}

	bProfile = false ;

	m_pEngine->InitEngine(bProfile) ;
	
	return true ;
}


CString CDraughtsEngineApp::GetPath( LPCTSTR FileName)
{
	int iFindChar ;
	CString csFileName, csPath ;

	csFileName.Format("%s", FileName) ;
	iFindChar	= csFileName.ReverseFind('\\') ;
	csPath		= csFileName.Left(iFindChar + 1) ;

	return csPath ;
}


BEGIN_MESSAGE_MAP ( CDraughtsDialog, CDialog )
	ON_BN_CLICKED	( IDC_ENGINE_CLEAR,	OnClearButtonClicked )
	ON_BN_CLICKED	( IDC_ENGINE_SEND,	OnSendButtonClicked )

	ON_MESSAGE		( WM_USER_GUIDE_MESSAGE, OnGUIDEMessage )
	
	ON_WM_PAINT()
	ON_WM_TIMER()

END_MESSAGE_MAP()


CDraughtsDialog::CDraughtsDialog()
{

	Create(IDD_DIALOG_DRAUGHTS) ;

	if ( !SetTimer(ID_DRAUGHTSENGINE_TIMER, 100, NULL ) ) {
		MessageBox("SetTimer failed","Error", MB_ICONSTOP | MB_OK ) ;
	}

	m_bThread = true ;
	
	InitOutputThread() ;
	InitGenerateThread() ;
	InitScriptThread() ;
	InitMatchThread() ;
	InitGameThread() ;
	InitSearchThread() ;
}


BOOL CDraughtsDialog::OnInitDialog()
{
	int i ;
	CHARFORMAT cfm ;
	CString csEngineName, csInstance ;

	CDialog::OnInitDialog() ;

	m_pApp			= (CDraughtsEngineApp*)AfxGetApp() ;
	m_pEngine		= m_pApp->m_pEngine ;
	m_bGuide		= m_pApp->m_bGuide ;
	m_pGuide		= m_pApp->m_pGuide ;
	m_pPDN			= m_pApp->m_pPDN ;
	m_bShowWindow	= m_pApp->m_bShowWindow ;

	m_pGuide->m_pWnd	= this ;

	m_bmBitmapBoard.LoadBitmap(IDB_BITMAP_EMPTYBOARD2) ;
	m_bmBitmapBoard.GetObject( sizeof(m_bmBoard), &m_bmBoard) ;
		
	m_bmBitmap[0].LoadBitmap(IDB_BITMAP_WHITEMAN2) ;	// Load Bitmaps
	m_bmBitmap[1].LoadBitmap(IDB_BITMAP_WHITEKING2) ;
	m_bmBitmap[2].LoadBitmap(IDB_BITMAP_BLACKMAN2) ;
	m_bmBitmap[3].LoadBitmap(IDB_BITMAP_BLACKKING2) ;

	m_bmBitmap[0].GetObject( sizeof(m_bmSquare), &m_bmSquare) ;

	for ( i=1 ; i<=50 ; i++ )
		m_iBoard[i] = m_pEngine->m_EngineData.iBoard[i] ;

	// Initiate PP LED's
	for ( i=0; i<4; i++ )
		m_LED_PP[i].SetLed(CLed::LED_COLOR_RED, CLed::LED_ON) ;

	// Initiate DXP LED's
	m_LED_DXP[0].SetLed(CLed::LED_COLOR_RED, CLed::LED_OFF) ;
	m_LED_DXP[1].SetLed(CLed::LED_COLOR_YELLOW, CLed::LED_OFF) ;
	m_LED_DXP[2].SetLed(CLed::LED_COLOR_GREEN, CLed::LED_OFF) ;
	m_LED_DXP[3].SetLed(CLed::LED_COLOR_BLUE, CLed::LED_OFF) ;
	m_LED_DXP[4].SetLed(CLed::LED_COLOR_BLUE, CLed::LED_OFF) ;

	// Initiate EMM LED's
	for ( i=0; i<8; i++ )
		m_LED_EMM[i].SetLed(CLed::LED_COLOR_GREEN, CLed::LED_OFF) ;

	// Change Default Font Engine_Output
	cfm.cbSize		= sizeof(cfm) ;
	cfm.dwMask		= CFM_SIZE | CFM_ITALIC | CFM_BOLD | CFM_UNDERLINE | CFM_STRIKEOUT | CFM_PROTECTED | CFM_FACE ;
	cfm.dwEffects	= 0 ;
	cfm.yHeight		= 40 ;

	strcpy_s(cfm.szFaceName, 16, "Courier") ;

	m_pOutput = (CRichEditCtrl*)GetDlgItem(IDC_ENGINE_OUTPUT) ;
	m_pOutput->SetDefaultCharFormat(cfm) ;

	return TRUE ;

}


void CDraughtsDialog::DoDataExchange(CDataExchange* pDX)
{

	CDialog::DoDataExchange(pDX) ;

	// LED_PP
	DDX_Control(pDX, IDC_STATIC_LED0, m_LED_PP[0] ) ;
	DDX_Control(pDX, IDC_STATIC_LED1, m_LED_PP[1] ) ;
	DDX_Control(pDX, IDC_STATIC_LED2, m_LED_PP[2] ) ;
	DDX_Control(pDX, IDC_STATIC_LED3, m_LED_PP[3] ) ;

	// LED_DXP
	DDX_Control(pDX, IDC_STATIC_LED4, m_LED_DXP[0] ) ;
	DDX_Control(pDX, IDC_STATIC_LED5, m_LED_DXP[1] ) ;
	DDX_Control(pDX, IDC_STATIC_LED6, m_LED_DXP[2] ) ;
	DDX_Control(pDX, IDC_STATIC_LED7, m_LED_DXP[3] ) ;
	DDX_Control(pDX, IDC_STATIC_LED8, m_LED_DXP[4] ) ;

	// LED_EMM
	DDX_Control(pDX, IDC_STATIC_LED9,  m_LED_EMM[0] ) ;
	DDX_Control(pDX, IDC_STATIC_LED10, m_LED_EMM[1] ) ;
	DDX_Control(pDX, IDC_STATIC_LED11, m_LED_EMM[2] ) ;
	DDX_Control(pDX, IDC_STATIC_LED12, m_LED_EMM[3] ) ;
	DDX_Control(pDX, IDC_STATIC_LED13, m_LED_EMM[4] ) ;
	DDX_Control(pDX, IDC_STATIC_LED14, m_LED_EMM[5] ) ;
	DDX_Control(pDX, IDC_STATIC_LED15, m_LED_EMM[6] ) ;
	DDX_Control(pDX, IDC_STATIC_LED16, m_LED_EMM[7] ) ;
}


void CDraughtsDialog::OnCancel()
{

	EngineQUIT() ;
}


void CDraughtsDialog::PostNcDestroy()
{

	delete this ;
}


void CDraughtsDialog::OnClearButtonClicked()
{
	int iTotalTextLength ;

	m_pOutput = (CRichEditCtrl*)GetDlgItem(IDC_ENGINE_OUTPUT) ;

	iTotalTextLength =	m_pOutput->GetWindowTextLength() ;
						m_pOutput->SetSel(0, iTotalTextLength) ;
						m_pOutput->ReplaceSel("") ;

}


void CDraughtsDialog::OnSendButtonClicked()
{	
	int iMessageID ;
	CString csCommand, csEngineCommand ;

	( (CEdit*)GetDlgItem(IDC_ENGINE_COMMAND) )->GetWindowText(csEngineCommand) ;
	( (CEdit*)GetDlgItem(IDC_ENGINE_COMMAND) )->SetWindowText("") ;

	if ( isdigit(csEngineCommand[0]) )		// Move command !
		csCommand.Format("%s %s", ENGINE_STR_MOVE, csEngineCommand) ;
	else
		csCommand = csEngineCommand ;
			
	iMessageID = m_pEngine->DecodeMessage(csCommand) ;
		
	if ( iMessageID >= 0 )
		m_pEngine->ExecuteMessage() ;

	if ( m_pEngine->BoardUpdate() )
		DisplayBoard() ;

	if ( iMessageID == ENGINE_QUIT )
		EngineQUIT() ;
}


void CDraughtsDialog::DisplayBoard()
{
	int i ;

	for ( i=1 ; i<=50 ; i++ )
		m_iBoard[i] = m_pEngine->m_EngineData.iBoard[i] ;

	InvalidateRect(m_rectBoard) ;
	UpdateWindow() ;
}


void CDraughtsDialog::DisplayMessage(int iOutput, CString csMessage)
{
	int iTextLength ;
	CString csDisplayMessage ;
	
	if ( iOutput >= 4 || iOutput < 0 )
		m_pOutput = (CRichEditCtrl*)GetDlgItem(IDC_ENGINE_OUTPUT) ;
	else if ( iOutput == 0 )
		m_pOutput = (CRichEditCtrl*)GetDlgItem(IDC_RICHEDIT_CORE0) ;
	else if ( iOutput == 1 )
		m_pOutput = (CRichEditCtrl*)GetDlgItem(IDC_RICHEDIT_CORE1) ;
	else if ( iOutput == 2 )
		m_pOutput = (CRichEditCtrl*)GetDlgItem(IDC_RICHEDIT_CORE2) ;
	else if ( iOutput == 3 )
		m_pOutput = (CRichEditCtrl*)GetDlgItem(IDC_RICHEDIT_CORE3) ;

	iTextLength = m_pOutput->GetTextLength() ;

	csDisplayMessage.Format("%s", csMessage) ;
	csDisplayMessage += "\r\n" ;
	
	m_pOutput->HideSelection(TRUE, TRUE) ;

	m_pOutput->SetSel(iTextLength, iTextLength) ;
	m_pOutput->ReplaceSel(csDisplayMessage) ;

	iTextLength = m_pOutput->GetTextLength() ;
	m_pOutput->SetSel(iTextLength, iTextLength) ;

	m_pOutput->HideSelection(FALSE, TRUE) ;
	m_pOutput->SendMessage(EM_SCROLLCARET, 0, 0) ;
}


bool CDraughtsDialog::ClearEngineOutput(int iOutput)
{
	int iTotalTextLength ;

	if ( iOutput >= 4 || iOutput < 0 )
		m_pOutput = (CRichEditCtrl*)GetDlgItem(IDC_ENGINE_OUTPUT) ;
	else if ( iOutput == 0 )
		m_pOutput = (CRichEditCtrl*)GetDlgItem(IDC_RICHEDIT_CORE0) ;
	else if ( iOutput == 1 )
		m_pOutput = (CRichEditCtrl*)GetDlgItem(IDC_RICHEDIT_CORE1) ;
	else if ( iOutput == 2 )
		m_pOutput = (CRichEditCtrl*)GetDlgItem(IDC_RICHEDIT_CORE2) ;
	else if ( iOutput == 3 )
		m_pOutput = (CRichEditCtrl*)GetDlgItem(IDC_RICHEDIT_CORE3) ;

	iTotalTextLength =	m_pOutput->GetWindowTextLength() ;
						m_pOutput->SetSel(0, iTotalTextLength) ;
						m_pOutput->ReplaceSel("") ;

	return ( true ) ;
}


void CDraughtsDialog::OnEngineModeStatusUpdate(void)
{
	int iEngineMode ;

	iEngineMode = m_pEngine->m_EngineData.iEngineMode ;

	m_LED_EMM[LED_MODE_N].SetLed(CLed::LED_COLOR_GREEN, iEngineMode ==  LED_MODE_N ? 0 : 1 ) ;
	m_LED_EMM[LED_MODE_A].SetLed(CLed::LED_COLOR_GREEN, iEngineMode ==  LED_MODE_A ? 0 : 1 ) ;
	m_LED_EMM[LED_MODE_G].SetLed(CLed::LED_COLOR_GREEN, iEngineMode ==  LED_MODE_G ? 0 : 1 ) ;
	m_LED_EMM[LED_MODE_M].SetLed(CLed::LED_COLOR_GREEN, iEngineMode ==  LED_MODE_M ? 0 : 1 ) ;
	m_LED_EMM[LED_MODE_L].SetLed(CLed::LED_COLOR_GREEN, iEngineMode ==  LED_MODE_L ? 0 : 1 ) ;
	m_LED_EMM[LED_MODE_B].SetLed(CLed::LED_COLOR_GREEN, iEngineMode ==  LED_MODE_B ? 0 : 1 ) ;
	m_LED_EMM[LED_MODE_E].SetLed(CLed::LED_COLOR_GREEN, iEngineMode ==  LED_MODE_E ? 0 : 1 ) ;
	m_LED_EMM[LED_MODE_I].SetLed(CLed::LED_COLOR_GREEN, iEngineMode ==  LED_MODE_I ? 0 : 1 ) ;

}


void CDraughtsDialog::OnEngineWindowDXPTextUpdate(void)
{
	CString csMessage ;

	csMessage.Format("DXP Match %s - %s: Win %d, Lose %d, Draw %d, Unknown %d", 
						m_pEngine->m_EngineData.DXPMatch.csClientName,  m_pEngine->m_EngineData.DXPMatch.csServerName,
						m_pEngine->m_EngineData.DXPMatch.iNWin, m_pEngine->m_EngineData.DXPMatch.iNLoose, 
						m_pEngine->m_EngineData.DXPMatch.iNDraw, m_pEngine->m_EngineData.DXPMatch.iNUnknown ) ;
	
	this->SetWindowText(csMessage) ;
}


void CDraughtsDialog::OnEngineWindowTextUpdate(void)
{
	CString csEngineName, csInstance ;

	csEngineName.Format("Engine : %s, Author : %s, Instance :", m_pEngine->m_EngineData.csEngineName,  m_pEngine->m_EngineData.csEngineAuthor) ;
	csInstance.Format(" [%.2d]", m_pApp->m_iInstanceCount) ;

	csEngineName += csInstance ;

	this->SetWindowText(csEngineName) ;
}


void CDraughtsDialog::OnStatusUpdate(void)
{
	int i, iStatus ;

	for ( i=0; i<PP_PROCESS_DISPLAY; i++ ) {

		iStatus = m_pEngine->m_EngineData.iPPProcessStatus[i] ;

		if ( iStatus == PP_PROCESS_NOTAVAILABLE )
			m_LED_PP[i].SetLed(CLed::LED_COLOR_RED, CLed::LED_ON) ;

		else if ( iStatus == PP_PROCESS_DEACTIVATED )
			m_LED_PP[i].SetLed(CLed::LED_COLOR_YELLOW, CLed::LED_ON) ;

		else if ( iStatus == PP_PROCESS_IDLE )
			m_LED_PP[i].SetLed(CLed::LED_COLOR_BLUE, CLed::LED_ON) ;

		else if ( iStatus == PP_PROCESS_RUN )
			m_LED_PP[i].SetLed(CLed::LED_COLOR_GREEN, CLed::LED_ON) ;
	}
}


LRESULT CDraughtsDialog::OnGUIDEMessage(WPARAM wParam, LPARAM lParam)
{
	int iIndex, iMessageID ;
	CString csCommand ;

	iIndex = (int)wParam ;

	if ( iIndex < 0 ) {	// Special cases related to DXP

		if ( iIndex == GUIDE_NOTIFICATION_ONACCEPT && m_pEngine->m_iCommunicationMode == ENGINE_COMMUNICATIONMODE_GUIDE ) {

			m_pEngine->OnOptions() ;
		}

		if ( iIndex == GUIDE_NOTIFICATION_ONACCEPT ) {

			m_pEngine->m_bClient = false ;
			m_pEngine->m_bServer = true ;

			m_LED_DXP[LED_DXP_CLIENT].SetLed(CLed::LED_COLOR_BLUE, CLed::LED_OFF) ;
			m_LED_DXP[LED_DXP_SERVER].SetLed(CLed::LED_COLOR_BLUE, CLed::LED_ON) ;

			m_LED_DXP[LED_DXP_CLOSE].SetLed(CLed::LED_COLOR_RED, CLed::LED_OFF) ;
			m_LED_DXP[LED_DXP_GAMEEND].SetLed(CLed::LED_COLOR_YELLOW, CLed::LED_OFF) ;
			m_LED_DXP[LED_DXP_GAMEACC].SetLed(CLed::LED_COLOR_GREEN, CLed::LED_OFF) ;

		}

		if ( iIndex == GUIDE_NOTIFICATION_ONCONNECT ) {

			m_pEngine->m_bClient = true ;
			m_pEngine->m_bServer = false ;

			m_LED_DXP[LED_DXP_CLIENT].SetLed(CLed::LED_COLOR_BLUE, CLed::LED_ON) ;
			m_LED_DXP[LED_DXP_SERVER].SetLed(CLed::LED_COLOR_BLUE, CLed::LED_OFF) ;

			m_LED_DXP[LED_DXP_CLOSE].SetLed(CLed::LED_COLOR_RED, CLed::LED_OFF) ;
			m_LED_DXP[LED_DXP_GAMEEND].SetLed(CLed::LED_COLOR_YELLOW, CLed::LED_OFF) ;
			m_LED_DXP[LED_DXP_GAMEACC].SetLed(CLed::LED_COLOR_GREEN, CLed::LED_OFF) ;

		}

		if ( iIndex == GUIDE_NOTIFICATION_ONCLOSE ) {

			m_pEngine->m_bClient = false ;
			m_pEngine->m_bServer = false ;

			m_LED_DXP[LED_DXP_CLOSE].SetLed(CLed::LED_COLOR_RED, CLed::LED_ON) ;
			m_LED_DXP[LED_DXP_GAMEEND].SetLed(CLed::LED_COLOR_YELLOW, CLed::LED_OFF) ;
			m_LED_DXP[LED_DXP_GAMEACC].SetLed(CLed::LED_COLOR_GREEN, CLed::LED_OFF) ;

		}

		if ( iIndex == GUIDE_NOTIFICATION_ONDXPGAMEACC ) {

			m_LED_DXP[LED_DXP_GAMEEND].SetLed(CLed::LED_COLOR_YELLOW, CLed::LED_OFF) ;
			m_LED_DXP[LED_DXP_GAMEACC].SetLed(CLed::LED_COLOR_GREEN, CLed::LED_ON) ;
		}

		if ( iIndex == GUIDE_NOTIFICATION_ONDXPGAMEEND ) {

			m_LED_DXP[LED_DXP_GAMEEND].SetLed(CLed::LED_COLOR_YELLOW, CLed::LED_ON) ;
			m_LED_DXP[LED_DXP_GAMEACC].SetLed(CLed::LED_COLOR_GREEN, CLed::LED_OFF) ;
		}

		return true ;
	}

	csCommand.Format("%s", m_pGuide->m_csGUIMessage[iIndex]) ;

	if ( m_pEngine->m_iCommunicationMode == ENGINE_COMMUNICATIONMODE_GUIDE ) {	// GUIDE Communication

		iMessageID = m_pEngine->DecodeMessage(csCommand) ;
		
			if ( iMessageID >= 0 )
			m_pEngine->ExecuteMessage() ;
	}

	if ( m_pEngine->m_iCommunicationMode == ENGINE_COMMUNICATIONMODE_DXP ) {	// DXP Communication

		iMessageID = m_pEngine->DecodeDXPMessage(csCommand) ;
		
			if ( iMessageID >= 0 )
			m_pEngine->ExecuteDXPMessage() ;
	}

	if ( m_pEngine->BoardUpdate() )
		DisplayBoard() ;

	if ( iMessageID == ENGINE_QUIT )
		EngineQUIT() ;

	return true ;
}


void CDraughtsDialog::EngineQUIT()
{
	exit(0) ;

//	m_bThread = false ;

//	KillTimer(ID_DRAUGHTSENGINE_TIMER) ;

//	DestroyWindow() ;

}


void CDraughtsDialog::OnPaint()
{
	int i, j, iIndex, iPos, iPSWidth, iType, iX, iXPos, iY, iYPos  ;
	int iOffsetX, iOffsetY ;
	int iBorderWidth ;
	CDC dcMem, dcMem2 ;


	GetDlgItem(IDC_STATIC_BOARD)->GetWindowRect(&m_rectBoard) ;

	ScreenToClient(&m_rectBoard) ;

	CPaintDC dcScreen( this ) ;

	dcMem.CreateCompatibleDC(&dcScreen) ;

	dcScreen.SetTextColor( RGB(0, 0, 0) ) ;
	dcScreen.SetBkColor( RGB(192, 192, 192) ) ;

	CBitmap* pOldBitmap = dcMem.SelectObject(&m_bmBitmapBoard) ;

	iOffsetX = ( m_rectBoard.Width() - m_bmBoard.bmWidth ) / 2 ;
	iOffsetY = ( m_rectBoard.Height() - m_bmBoard.bmHeight ) / 2 ;

	dcScreen.BitBlt(m_rectBoard.left + iOffsetX, m_rectBoard.top + iOffsetY, m_bmBoard.bmWidth, m_bmBoard.bmHeight, &dcMem, 0, 0, SRCCOPY ) ;
	dcMem.SelectObject(pOldBitmap) ;

	iPSWidth		= m_bmSquare.bmWidth ;
	iBorderWidth	= ( m_bmBoard.bmWidth - ( 10 * m_bmSquare.bmWidth ) ) / 2 ;

	for ( j=0; j<10; j++ )
		for ( i=0; i<5; i++ ) {

			iPos = ( 5 * j ) + i + 1 ;
			iType = m_iBoard[iPos] ;

			iIndex = -1 ;

			if ( iType == PS_WHITEMAN )
				iIndex = 0 ;
			else if ( iType == PS_WHITEKING )
				iIndex = 1 ;
			else if ( iType == PS_BLACKMAN )
				iIndex = 2 ;
			else if ( iType == PS_BLACKKING )
				iIndex = 3 ;
	
			if ( iIndex >= 0 ) {

				iX = ( 2 * i ) + (j + 1)%2 ;
				iY = j ;

				iXPos = iOffsetX + m_rectBoard.left + iPSWidth * iX + iBorderWidth  ;
				iYPos = iOffsetY + m_rectBoard.top  + iPSWidth * iY + iBorderWidth ;
	
				CBitmap* pOldBitmap = dcMem.SelectObject(&m_bmBitmap[iIndex]) ;
				dcScreen.BitBlt(iXPos, iYPos, m_bmSquare.bmWidth-1, m_bmSquare.bmHeight-1, &dcMem, 0, 0, SRCCOPY ) ;
				dcMem.SelectObject(pOldBitmap) ;
			}
	}
}


void CDraughtsDialog::OnTimer(UINT_PTR nTimerID )
{
	unsigned int dTime ;
	PENGINEDATA pEngineData ;

	pEngineData = &(m_pEngine->m_EngineData) ;

	if (	pEngineData->bSearch && 
			( pEngineData->iSearchMode == ENGINE_SEARCH_FIXED_TIME || pEngineData->iSearchMode == ENGINE_SEARCH_BLITZGAME ) && 
			pEngineData->iEngineMode != ENGINE_MODE_INFINITE_ANALYSIS &&
			pEngineData->bPanicTime == false ) {

		dTime = GetTickCount() ;

		pEngineData->dEngineSearchTime = ( dTime - m_pEngine->m_EngineData.dTimeSearch )/1000 ;

		if ( pEngineData->dEngineSearchTime >= pEngineData->iSearchValue )
			m_pEngine->Halt(0) ;
 
	}
}


void CDraughtsDialog::InitOutputThread()
{

	m_pOutputThread = AfxBeginThread(OutputThreadFunc, this) ;

}


UINT CDraughtsDialog::OutputThreadFunc(LPVOID me)
{
	CDraughtsDialog* self = (CDraughtsDialog*)me ;
	self->runOutput() ;

	return ( 0 ) ;
}


void CDraughtsDialog::runOutput()
{
	bool bEngineSound = false ;
	int iIndex ;
	OUTPUTSTACK OutputStack ;

	while ( m_bThread ) {

		if ( m_pEngine->m_EngineData.iOutputStackRead == m_pEngine->m_EngineData.iOutputStackWrite )
			Sleep(10) ;
		else {
			iIndex = m_pEngine->m_EngineData.iOutputStackRead % ENGINE_OUTPUT_STACK_SIZE ;

			OutputStack = m_pEngine->m_EngineData.OutputStack[iIndex] ;

			if ( OutputStack.iMessageID == ENGINE_$STATUS ) {
				OnStatusUpdate() ;

			} else if ( OutputStack.iMessageID == ENGINE_NOTIFICATION_ONUPDATE_ENGINEMODE ) {
				OnEngineModeStatusUpdate() ;

			} else if ( OutputStack.iMessageID == ENGINE_NOTIFICATION_ONUPDATE_ENGINEWINDOWTEXT ) {
				OnEngineWindowTextUpdate() ;

			} else {

				if ( ( OutputStack.iMessageSource != ENGINE_CONSOLE_INFO ) && 
					( m_pEngine->m_iCommunicationMode == ENGINE_COMMUNICATIONMODE_GUIDE || 
					OutputStack.iMessageID == ENGINE_DXP_MESSAGE || 
					OutputStack.iMessageID == ENGINE_DXPMOVE_MESSAGE || 
					OutputStack.iMessageID == ENGINE_DXPGAMEREQ_MESSAGE ) )
					m_pGuide->SendMessageToGUI(OutputStack.csOutputMessage ) ;
			
				if ( m_bShowWindow ) {

					if ( OutputStack.iMessageID == ENGINE_$CLEAROUTPUT )
						ClearEngineOutput(OutputStack.iMessageSource) ;			// Clear EngineOutput Window 
					else
						DisplayMessage(OutputStack.iMessageSource, OutputStack.csOutputMessage) ;
			
					if ( OutputStack.iMessageID == ENGINE_QMRK_DISPLAYBOARD )
						DisplayBoard() ;

					if ( OutputStack.iMessageID == ENGINE_MOVE ) {
						DisplayBoard() ;

						if ( bEngineSound )
							Beep(1000,500) ;
					}

					if ( OutputStack.iMessageID == ENGINE_DXPMOVE_MESSAGE || OutputStack.iMessageID == ENGINE_DXPGAMEREQ_MESSAGE ) {
						DisplayBoard() ;
					}
				}
			}

			if ( m_pEngine->m_EngineData.iEngineMode == ENGINE_MODE_BOOK_UPDATE && OutputStack.iMessageID == ENGINE_QMRK_BOOKUPDATE )
				if ( m_pEngine->BookUpdateNext() )
					m_pEngine->Go() ;

			++(m_pEngine->m_EngineData.iOutputStackRead) ;
		}
	}
}

void CDraughtsDialog::InitSearchThread()
{
	int i, iNCore ;

	iNCore	= omp_get_num_procs() ;

	m_iNSearchThread = iNCore ;

	for ( i=0; i<SEARCHTHREAD_MAXTHREAD; i++ )
		m_iSearchThreadID[i] = 0 ;

	// All threads except the main thread should be initialized to idle state
	for ( i=1; i<iNCore; i++ ) {

		m_pEngine->m_EngineData.SearchThread[i].bStop			= false ;
		m_pEngine->m_EngineData.SearchThread[i].bWorkIsWaiting	= false ;
		m_pEngine->m_EngineData.SearchThread[i].bIdle			= true ;
		m_pEngine->m_EngineData.SearchThread[i].bRunning		= false ;
	}

	for ( i=0; i<iNCore; i++ ) {

		m_pEngine->m_EngineData.hEventStart[i]		= ::CreateEvent(NULL, false, false, NULL ) ;
		m_pEngine->m_EngineData.hEventStop[i]		= ::CreateEvent(NULL, true, false, NULL ) ;
		m_pEngine->m_EngineData.hSitIdleEvent[i]	= ::CreateEvent(NULL, false, false, NULL ) ;

		m_pSearchThread[i] = AfxBeginThread(SearchThreadFunc, this) ;

		m_pEngine->m_EngineData.iPPProcessStatus[i] = PP_PROCESS_IDLE ;
	}

	m_pEngine->m_EngineData.hEventThreadReady	= ::CreateEvent(NULL, false, false, NULL ) ;

	m_pEngine->m_hEventBookStart				= ::CreateEvent(NULL, false, false, NULL ) ;
	m_pEngine->m_EngineData.hEventSearchStop	= ::CreateEvent(NULL, false, false, NULL ) ;

	OnStatusUpdate() ;
}


UINT CDraughtsDialog::SearchThreadFunc(LPVOID me)
{	
	CDraughtsDialog* self = (CDraughtsDialog*)me ;

	self->runSearch() ;

	return ( 0 ) ;
}


void CDraughtsDialog::runSearch(void)
{
	bool bRunPostProcSearch ;
	int i, iRunSearch, iSearchThreadID, iThreadID ;

	iThreadID		= ::GetCurrentThreadId() ;

	iSearchThreadID = -1 ;

	for ( i=0; i<SEARCHTHREAD_MAXTHREAD; i++ ) {

		if ( m_iSearchThreadID[i] == 0 ) {

			m_iSearchThreadID[i]	= iThreadID ;
			iSearchThreadID			= i ;
			break ;
		}
	}

	while ( 1 ) {

		::WaitForSingleObject( m_pEngine->m_EngineData.hEventStart[iSearchThreadID], INFINITE) ;
			
		// Run Initial Search
		iRunSearch = m_pEngine->runSearch(iSearchThreadID) ;

		SetEvent( m_pEngine->m_EngineData.hEventStop[iSearchThreadID] ) ;

		if ( iSearchThreadID != m_pEngine->m_EngineData.iPPMasterThread )
			SetEvent( m_pEngine->m_EngineData.hEventThreadReady ) ; 

		bRunPostProcSearch = false ;

		switch ( m_pEngine->m_EngineData.iPPMode ) {

			case PP_MODE_SINGLECORE:
				bRunPostProcSearch = true ;
				break ;

			case PP_MODE_MULTICORE_COPYSEARCH:
				::WaitForMultipleObjects(m_pEngine->m_EngineData.iPPProcessActive, m_pEngine->m_EngineData.hEventStop, true, INFINITE) ;
				if ( iSearchThreadID == 0 )
					bRunPostProcSearch = true ;
				break ;

			case PP_MODE_MULTICORE_SMP_SEARCH:
//				::WaitForMultipleObjects(m_pEngine->m_EngineData.iPPProcessActive + 1, m_pEngine->m_EngineData.hEventStop, true, INFINITE) ;
				if ( iSearchThreadID == m_pEngine->m_EngineData.iPPMasterThread )
					bRunPostProcSearch = true ;
				break ;
		
		}

		// Run Post processing Search
		if ( bRunPostProcSearch ) {

			m_pEngine->runPostProcSearch(iSearchThreadID, iRunSearch) ;
			
			SetEvent( m_pEngine->m_EngineData.hEventSearchStop ) ;
		}

		Sleep(5) ;

	}
}


void CDraughtsDialog::InitGenerateThread()
{

	m_pEngine->m_hEventGenerateStart = ::CreateEvent(NULL, false, false, NULL ) ;

	m_pGenerateThread	= AfxBeginThread(GenerateThreadFunc, this) ;

}


UINT CDraughtsDialog::GenerateThreadFunc(LPVOID me)
{

	CDraughtsDialog* self = (CDraughtsDialog*)me ;
	self->runGenerate() ;

	return ( 0 ) ;
}


void CDraughtsDialog::runGenerate(void)
{

	while ( 1 ) {

		::WaitForSingleObject( m_pEngine->m_hEventGenerateStart, INFINITE) ;

		switch ( m_pEngine->m_iGenerateMode ) {

			case ENGINE_GENERATE_MODE_NONE:
				break ;

			case ENGINE_GENERATE_MODE_OPENINGBOOK:
				m_pEngine->runOpeningBookGenerate() ;
				break ;

			case ENGINE_GENERATE_MODE_ENDGAMEDATABASES:		
				m_pEngine->runEndGameDatabasesGenerate() ;
				break ;
		}

		SetEvent( m_pEngine->m_EngineData.hEventStop[0] ) ;
		Sleep(5) ;

	}
}


void CDraughtsDialog::InitScriptThread()
{

	m_pEngine->m_hEventScriptStart = ::CreateEvent(NULL, false, false, NULL ) ;

	m_pScriptThread	= AfxBeginThread(ScriptThreadFunc, this) ;

}


UINT CDraughtsDialog::ScriptThreadFunc(LPVOID me)
{

	CDraughtsDialog* self = (CDraughtsDialog*)me ;
	self->runScript() ;

	return ( 0 ) ;
}


void CDraughtsDialog::runScript(void)
{
	bool bContinue ;
	int iCommand, iNCommand, iMessageID ;
	DWORD dwReturnCode ;
	CString csCommand ;

	while ( 1 ) {

		::WaitForSingleObject( m_pEngine->m_hEventScriptStart, INFINITE) ;
			
		// Execute Script
		bContinue	= true ;
		iCommand	= 0 ;
		iNCommand	= m_pEngine->m_Script.iNCommand ;

		while ( bContinue ) {

			csCommand = m_pEngine->m_Script.csCommand[iCommand] ;

			iMessageID = m_pEngine->DecodeMessage(csCommand) ;
		
			if ( iMessageID >= 0 )
				m_pEngine->ExecuteMessage() ;

			if ( iMessageID == ENGINE_GO )	// Need to wait for Search-Thread to finish
				dwReturnCode = ::WaitForSingleObject( m_pEngine->m_EngineData.hEventSearchStop, INFINITE) ;

			if ( m_pEngine->BoardUpdate() )
				DisplayBoard() ;

			++iCommand ;

			if ( iCommand == iNCommand )
				bContinue = false ;

			Sleep(5) ;
		}
	}
}


void CDraughtsDialog::InitGameThread()
{

	m_pEngine->m_hEventGameStart	= ::CreateEvent(NULL, false, false, NULL ) ;
	m_pEngine->m_hEventGameStop		= ::CreateEvent(NULL, true, false, NULL ) ;

	m_pGameThread = AfxBeginThread(GameThreadFunc, this) ;

}


UINT CDraughtsDialog::GameThreadFunc(LPVOID me)
{

	CDraughtsDialog* self = (CDraughtsDialog*)me ;
	self->runGame() ;

	return ( 0 ) ;
}


void CDraughtsDialog::runGame(void)
{
	int iGameHistoryActualMove, iGameHistoryMaxMove ;
	DWORD dwReturnCode ;

	while ( 1 ) {

		::WaitForSingleObject( m_pEngine->m_hEventGameStart, INFINITE) ;
			
		// Execute Game
		iGameHistoryActualMove	= 0 ;
		iGameHistoryMaxMove		= m_pEngine->m_EngineData.iGameHistoryMaxMove ;

		while ( iGameHistoryActualMove < iGameHistoryMaxMove ) {

			m_pEngine->JumpTo(iGameHistoryActualMove) ;

			if ( m_pEngine->BoardUpdate() )
				DisplayBoard() ;

			m_pEngine->Go() ;

			dwReturnCode = ::WaitForSingleObject( m_pEngine->m_EngineData.hEventSearchStop, INFINITE) ;

			++iGameHistoryActualMove ;

			Sleep(5) ;
		}
	}
}


void CDraughtsDialog::InitMatchThread()
{

	m_pEngine->m_hEventMatchStart = ::CreateEvent(NULL, false, false, NULL ) ;

	m_pMatchThread = AfxBeginThread(MatchThreadFunc, this) ;

}


UINT CDraughtsDialog::MatchThreadFunc(LPVOID me)
{

	CDraughtsDialog* self = (CDraughtsDialog*)me ;
	self->runMatch() ;

	return ( 0 ) ;
}


void CDraughtsDialog::runMatch(void)
{
	BOOL bResetEvent ;
	int iMaxGames, iNGames ;
	DWORD dwReturnCode ;


	while ( 1 ) {
	
		::WaitForSingleObject( m_pEngine->m_hEventMatchStart, INFINITE) ;
		
		iNGames		= m_pEngine->m_EngineData.DXPMatch.iNGames ;
		iMaxGames	= m_pEngine->m_EngineData.DXPMatch.iMaxGames ;

		OnEngineWindowDXPTextUpdate() ;

		while ( iNGames < iMaxGames ) {

			// Execute Game
			m_pEngine->m_EngineData.DXPMatch.iGameResult = DXP_UNKNOWN ;

			m_pEngine->DXPBallotGame(iNGames) ;

			m_pEngine->DamageTime(DXP_MATCH_GAME_START) ;

			m_pEngine->DamageLog(DXP_MATCH_LOG_GAME_START) ;

			m_pEngine->DXPClientStart() ;

			dwReturnCode = ::WaitForSingleObject( m_pEngine->m_hEventGameStop, INFINITE) ;

			bResetEvent = ResetEvent(m_pEngine->m_hEventGameStop) ;

			m_pEngine->DamageTime(DXP_MATCH_GAME_END) ;

			m_pEngine->DXPUpdateStat(iNGames) ;		// Update stats
			
			m_pEngine->DamageLog(DXP_MATCH_LOG_GAME_END) ;

			++iNGames ;
			m_pEngine->m_EngineData.DXPMatch.iNGames = iNGames ;

			if ( m_pEngine->m_EngineData.DXPMatch.bPDN )
				m_pPDN->Write_PDNGame(&m_pEngine->m_EngineData) ;
			
			m_pEngine->DXPStat() ;					// Display stats

			m_pEngine->DamageLog(DXP_MATCH_LOG_STAT) ;

			OnEngineWindowDXPTextUpdate() ;

			Sleep(1000) ;
		}
	}
}