// Definition AppName
#define APPNAME						"Damage 11.2 Engine"

// Definition Timer
#define ID_DRAUGHTSENGINE_TIMER		1

// Definitions Directory Paths
#define PATH_DIRECTORY_BOOK			"Book\\"
#define PATH_DIRECTORY_DATABASES	"Databases\\"
#define PATH_DIRECTORY_GAMES		"Games\\"
#define PATH_DIRECTORY_SCRIPTS		"Scripts\\"
#define PATH_DIRECTORY_TABLES		"Tables\\"

// Definition Board Pieces
#define PS_WHITEMAN		1
#define PS_WHITEKING	2
#define PS_BLACKMAN		-1
#define PS_BLACKKING	-2
#define PS_EMPTY		0

// Definition Process Displays
#define PP_PROCESS_DISPLAY	4

// Definition Status LEDS DXP
#define LED_DXP_CLOSE		0
#define LED_DXP_GAMEEND		1
#define LED_DXP_GAMEACC		2
#define LED_DXP_SERVER		3
#define LED_DXP_CLIENT		4

// Definition Status LEDS EMM
#define LED_MODE_N		0
#define LED_MODE_A		1
#define LED_MODE_G		2
#define LED_MODE_M		3
#define LED_MODE_L		4
#define LED_MODE_B		5
#define LED_MODE_E		6
#define LED_MODE_I		7

class CDraughtsEngineApp : public CWinApp
{

public:

	bool	m_bGuide ;
	bool	m_bShowWindow ;

	int		m_iCommunicationMode ;
	int		m_iInstanceCount ;

	CString m_csApplicationPath ;
	CString	m_csBookPath ;
	CString m_csDatabasePath ;
	CString m_csGamesPath ;
	CString m_csScriptsPath ;
	CString m_csTablesPath ;

	CEngine*	m_pEngine ;	// Pointer to CEngine Class
	CGuide*		m_pGuide ;	// Pointer to CGuide Class
	CPDN*		m_pPDN ;	// Pointer to CPDN Class
	
	virtual BOOL InitInstance() ;
	
	CString GetPath( LPCTSTR ) ;

} ;


class CDraughtsDialog : public CDialog
{

public:

	CLed		m_LED_PP[PP_PROCESS_DISPLAY], m_LED_DXP[5], m_LED_EMM[8] ;

	CDraughtsDialog() ;

	BOOL			OnInitDialog() ;	
	
	void			OnEngineModeStatusUpdate(void) ;
	void			OnEngineWindowDXPTextUpdate(void) ;
	void			OnEngineWindowTextUpdate(void) ;
	void			OnStatusUpdate(void) ;

	LRESULT			OnGUIDEMessage(WPARAM, LPARAM) ;

private:
	bool		m_bGuide ;
	bool		m_bShowWindow ;
	bool		m_bThread ;

	int			m_iBoard[51] ;

	int			m_iNSearchThread ;
	int			m_iSearchThreadID[SEARCHTHREAD_MAXTHREAD] ;

	CWinThread* m_pOutputThread ;
	CWinThread*	m_pSearchThread[SEARCHTHREAD_MAXTHREAD] ;
	CWinThread* m_pGenerateThread ;
	CWinThread* m_pScriptThread ;
	CWinThread* m_pGameThread ;
	CWinThread* m_pMatchThread ;
	
	CDraughtsEngineApp* m_pApp ;

	CEngine*		m_pEngine ;	// Pointer to CEngine Class
	CGuide*			m_pGuide ;	// Pointer to CGuide Class
	CPDN*			m_pPDN ;	// Pointer to CPDN Class
	
	CRichEditCtrl*	m_pOutput ;	// Pointer to CRichEditCtrl Class
	
	CRect			m_rectBoard ;
	CBitmap			m_bmBitmapBoard, m_bmBitmap[4] ;
	BITMAP			m_bmSquare, m_bmBoard ;

	void DisplayBoard(void) ;
	void DisplayMessage(int, CString) ;
	bool ClearEngineOutput(int) ;

	void EngineQUIT(void) ;

	void InitOutputThread(void) ;
	static UINT OutputThreadFunc(LPVOID) ;
	void runOutput(void) ;
	
	void InitSearchThread(void) ;
	static UINT SearchThreadFunc(LPVOID) ;
	void	runSearch(void) ;

	void InitGenerateThread(void) ;
	static UINT GenerateThreadFunc(LPVOID) ;
	void	runGenerate(void) ;

	void InitScriptThread(void) ;
	static UINT ScriptThreadFunc(LPVOID) ;
	void	runScript(void) ;

	void InitGameThread(void) ;
	static UINT GameThreadFunc(LPVOID) ;
	void	runGame(void) ;

	void InitMatchThread(void) ;
	static UINT MatchThreadFunc(LPVOID) ;
	void	runMatch(void) ;

protected:
	virtual void DoDataExchange(CDataExchange*) ;
	virtual void OnCancel() ;
	virtual void PostNcDestroy() ;

	afx_msg void OnClearButtonClicked() ;
	afx_msg void OnSendButtonClicked() ;

	afx_msg void OnPaint() ;
	afx_msg void OnTimer(UINT_PTR) ;

	DECLARE_MESSAGE_MAP()

} ;